@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 # dbus.rb - Module containing the low-level D-Bus implementation
 #
 # This file is part of the ruby-dbus project
@@ -8,6 +10,8 @@
 # License, version 2.1 as published by the Free Software Foundation.
 # See the file "COPYING" for the exact licensing terms.
 
+require_relative "raw_message"
+
 # = D-Bus main module
 #
 # Module containing all the D-Bus modules and classes.
@@ -27,7 +31,7 @@ module DBus
     # Mutex that protects updates on the serial number.
     @@serial_mutex = Mutex.new
     # Type of a message (by specification).
-    MESSAGE_SIGNATURE = "yyyyuua(yv)".freeze
+    MESSAGE_SIGNATURE = "yyyyuua(yv)"
 
     # FIXME: following message type constants should be under Message::Type IMO
     # well, yeah sure
@@ -43,6 +47,9 @@ module DBus
     # Signal message type.
     SIGNAL = 4
 
+    # Names used by signal match rules
+    TYPE_NAMES = ["invalid", "method_call", "method_return", "error", "signal"].freeze
+
     # Message flag signyfing that no reply is expected.
     NO_REPLY_EXPECTED = 0x1
     # Message flag signifying that no automatic start is required/must be
@@ -66,11 +73,13 @@ module DBus
     attr_accessor :sender
     # The signature of the message contents.
     attr_accessor :signature
-    # The serial number of the message this message is a reply for.
+    # @return [Integer] (u32)
+    #   The serial number of the message this message is a reply for.
     attr_accessor :reply_serial
     # The protocol.
     attr_reader :protocol
-    # The serial of the message.
+    # @return [Integer] (u32)
+    #   The serial of the message.
     attr_reader :serial
     # The parameters of the message.
     attr_reader :params
@@ -105,22 +114,28 @@ module DBus
       "error_name=#{error_name}"
     end
 
-    # Create a regular reply to a message _m_.
-    def self.method_return(m)
-      MethodReturnMessage.new.reply_to(m)
+    # @return [String] name of message type, as used in match rules:
+    #    "method_call", "method_return", "signal", "error"
+    def message_type_s
+      TYPE_NAMES[message_type] || "unknown_type_#{message_type}"
     end
 
-    # Create an error reply to a message _m_.
-    def self.error(m, error_name, description = nil)
-      ErrorMessage.new(error_name, description).reply_to(m)
+    # Create a regular reply to a message _msg_.
+    def self.method_return(msg)
+      MethodReturnMessage.new.reply_to(msg)
     end
 
-    # Mark this message as a reply to a another message _m_, taking
-    # the serial number of _m_ as reply serial and the sender of _m_ as
+    # Create an error reply to a message _msg_.
+    def self.error(msg, error_name, description = nil)
+      ErrorMessage.new(error_name, description).reply_to(msg)
+    end
+
+    # Mark this message as a reply to a another message _msg_, taking
+    # the serial number of _msg_ as reply serial and the sender of _msg_ as
     # destination.
-    def reply_to(m)
-      @reply_serial = m.serial
-      @destination = m.sender
+    def reply_to(msg)
+      @reply_serial = msg.serial
+      @destination = msg.sender
       self
     end
 
@@ -131,6 +146,10 @@ module DBus
       @params << [type, val]
     end
 
+    # "l" or "B"
+    ENDIANNESS_CHAR = ENV.fetch("RUBY_DBUS_ENDIANNESS", HOST_END)
+    ENDIANNESS = RawMessage.endianness(ENDIANNESS_CHAR)
+
     # FIXME: what are these? a message element constant enumeration?
     # See method below, in a message, you have and array of optional parameters
     # that come with an index, to determine their meaning. The values are in
@@ -152,14 +171,14 @@ module DBus
         raise InvalidDestinationName
       end
 
-      params = PacketMarshaller.new
-      @params.each do |param|
-        params.append(param[0], param[1])
+      params_marshaller = PacketMarshaller.new(endianness: ENDIANNESS)
+      @params.each do |type, value|
+        params_marshaller.append(type, value)
       end
-      @body_length = params.packet.bytesize
+      @body_length = params_marshaller.packet.bytesize
 
-      marshaller = PacketMarshaller.new
-      marshaller.append(Type::BYTE, HOST_END)
+      marshaller = PacketMarshaller.new(endianness: ENDIANNESS)
+      marshaller.append(Type::BYTE, ENDIANNESS_CHAR.ord)
       marshaller.append(Type::BYTE, @message_type)
       marshaller.append(Type::BYTE, @flags)
       marshaller.append(Type::BYTE, @protocol)
@@ -178,10 +197,8 @@ module DBus
       marshaller.append("a(yv)", headers)
 
       marshaller.align(8)
-      @params.each do |param|
-        marshaller.append(param[0], param[1])
-      end
-      marshaller.packet
+
+      marshaller.packet + params_marshaller.packet
     end
 
     # Unmarshall a packet contained in the buffer _buf_ and set the
@@ -191,13 +208,7 @@ module DBus
     #   the detected message (self) and
     #   the index pointer of the buffer where the message data ended.
     def unmarshall_buffer(buf)
-      buf = buf.dup
-      endianness = if buf[0] == "l"
-                     LIL_END
-                   else
-                     BIG_END
-                   end
-      pu = PacketUnmarshaller.new(buf, endianness)
+      pu = PacketUnmarshaller.new(buf, RawMessage.endianness(buf[0]))
       mdata = pu.unmarshall(MESSAGE_SIGNATURE)
       _, @message_type, @flags, @protocol, @body_length, @serial,
         headers = mdata
@@ -222,21 +233,21 @@ module DBus
           @signature = struct[1]
         end
       end
-      pu.align(8)
-      if @body_length > 0 && @signature
+      pu.align_body
+      if @body_length.positive? && @signature
         @params = pu.unmarshall(@signature, @body_length)
       end
-      [self, pu.idx]
+      [self, pu.consumed_size]
     end
 
     # Make a new exception from ex, mark it as being caused by this message
     # @api private
-    def annotate_exception(ex)
-      new_ex = ex.exception("#{ex}; caused by #{self}")
-      new_ex.set_backtrace(ex.backtrace)
-      new_ex
+    def annotate_exception(exc)
+      new_exc = exc.exception("#{exc}; caused by #{self}")
+      new_exc.set_backtrace(exc.backtrace)
+      new_exc
     end
-  end # class Message
+  end
 
   class MethodReturnMessage < Message
     def initialize
@@ -251,17 +262,17 @@ module DBus
       add_param(Type::STRING, description) unless description.nil?
     end
 
-    def self.from_exception(ex)
-      name = if ex.is_a? DBus::Error
-               ex.name
+    def self.from_exception(exc)
+      name = if exc.is_a? DBus::Error
+               exc.name
              else
                "org.freedesktop.DBus.Error.Failed"
-               # ex.class.to_s # RuntimeError is not a valid name, has no dot
+               # exc.class.to_s # RuntimeError is not a valid name, has no dot
              end
-      description = ex.message
+      description = exc.message
       msg = new(name, description)
-      msg.add_param(DBus.type("as"), ex.backtrace)
+      msg.add_param(DBus.type("as"), exc.backtrace)
       msg
     end
   end
-end # module DBus
+end
